@@ -1,18 +1,21 @@
 # -*- coding: utf-8 -*-
-from copy import deepcopy
 import itertools
+from copy import deepcopy
+
+import networkx as nx
+from networkx import MultiGraph
+
 from bg.edge import BGEdge, BGEdge_JSON_SCHEMA_JSON_KEY
 from bg.genome import BGGenome, BGGenome_JSON_SCHEMA_JSON_KEY
 from bg.kbreak import KBreak
 from bg.multicolor import Multicolor
+from bg.utils import get_from_dict_with_path, merge_fragment_edge_data, recursive_dict_update
 from bg.vertices import BGVertex_JSON_SCHEMA_JSON_KEY, BlockVertex, BGVertex, InfinityVertex, TaggedInfinityVertex, \
     TaggedBlockVertex, TaggedVertex
 
 __author__ = "Sergey Aganezov"
 __email__ = "aganezov(at)gwu.edu"
 __status__ = "production"
-from networkx import MultiGraph
-import networkx as nx
 
 
 class BreakpointGraph(object):
@@ -67,6 +70,8 @@ class BreakpointGraph(object):
         :param graph: is supplied, :class:`BreakpointGraph` is initialized with supplied or brand new (empty) instance of NetworkX MultiGraph.
         :type graph: instance of NetworkX MultiGraph is expected.
         """
+        self.cache = {}
+        self.cache_valid = {}
         if graph is None:
             self.bg = MultiGraph()
         else:
@@ -85,7 +90,7 @@ class BreakpointGraph(object):
         :rtype: ``generator``
         """
         for v1, v2, key, data in self.bg.edges_iter(nbunch=nbunch, data=True, keys=True):
-            bgedge = BGEdge(vertex1=v1, vertex2=v2, multicolor=data["multicolor"])
+            bgedge = BGEdge(vertex1=v1, vertex2=v2, multicolor=data["multicolor"], data=data["data"])
             if not keys:
                 yield bgedge
             else:
@@ -103,7 +108,8 @@ class BreakpointGraph(object):
         :return: generator over edges in current :class:`BreakpointGraph`
         :rtype: ``generator``
         """
-        yield from self.__edges(nbunch=nbunch, keys=keys)
+        for entry in self.__edges(nbunch=nbunch, keys=keys):
+            yield entry
 
     def nodes(self):
         """ Iterates over nodes in current :class:`BreakpointGraph` instance.
@@ -111,9 +117,10 @@ class BreakpointGraph(object):
         :return:  generator over nodes (vertices) in current :class:`BreakpointGraph` instance.
         :rtype: ``generator``
         """
-        yield from self.bg.nodes_iter()
+        for entry in self.bg.nodes_iter():
+            yield entry
 
-    def add_edge(self, vertex1, vertex2, multicolor, merge=True):
+    def add_edge(self, vertex1, vertex2, multicolor, merge=True, data=None):
         """ Creates a new :class:`bg.edge.BGEdge` object from supplied information and adds it to current instance of :class:`BreakpointGraph`.
 
         Proxies a call to :meth:`BreakpointGraph._BreakpointGraph__add_bgedge` method.
@@ -128,7 +135,7 @@ class BreakpointGraph(object):
         :type merge: ``Boolean``
         :return: ``None``, performs inplace changes
         """
-        self.__add_bgedge(BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=multicolor), merge=merge)
+        self.__add_bgedge(BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=multicolor, data=data), merge=merge)
 
     def __add_bgedge(self, bgedge, merge=True):
         """ Adds supplied :class:`bg.edge.BGEdge` object to current instance of :class:`BreakpointGraph`.
@@ -144,8 +151,11 @@ class BreakpointGraph(object):
         if bgedge.vertex1 in self.bg and bgedge.vertex2 in self.bg[bgedge.vertex1] and merge:
             key = min(self.bg[bgedge.vertex1][bgedge.vertex2].keys())
             self.bg[bgedge.vertex1][bgedge.vertex2][key]["multicolor"] += bgedge.multicolor
+            self.bg[bgedge.vertex1][bgedge.vertex2][key]["data"] = {}
         else:
-            self.bg.add_edge(u=bgedge.vertex1, v=bgedge.vertex2, attr_dict={"multicolor": deepcopy(bgedge.multicolor)})
+            self.bg.add_edge(u=bgedge.vertex1, v=bgedge.vertex2, attr_dict={"multicolor": deepcopy(bgedge.multicolor),
+                                                                            "data": bgedge.data})
+        self.cache_valid["overall_set_of_colors"] = False
 
     def add_bgedge(self, bgedge, merge=True):
         """ Adds supplied :class:`bg.edge.BGEdge` object to current instance of :class:`BreakpointGraph`.
@@ -170,11 +180,30 @@ class BreakpointGraph(object):
         :return: vertex with supplied label if present in current :class:`BreakpointGraph`, ``None`` otherwise
         """
         vertex_class = BGVertex.get_vertex_class_from_vertex_name(vertex_name)
-        name = BGVertex.get_vertex_name_root(vertex_name)
-        result = vertex_class(name)
+        data = vertex_name.split(BlockVertex.NAME_SEPARATOR)
+        root_name, data = data[0], data[1:]
+        if issubclass(vertex_class, TaggedVertex):
+            tags = [entry.split(TaggedVertex.TAG_SEPARATOR) for entry in data]
+            for tag_entry in tags:
+                if len(tag_entry) == 1:
+                    tag_entry.append(None)
+                elif len(tag_entry) > 2:
+                    tag_entry[1:] = [TaggedVertex.TAG_SEPARATOR.join(tag_entry[1:])]
+            result = vertex_class(root_name)
+            for tag, value in tags:
+                if tag == InfinityVertex.NAME_SUFFIX and issubclass(vertex_class, InfinityVertex):
+                    continue
+                result.add_tag(tag, value)
+        else:
+            result = vertex_class(root_name)
+
         if result in self.bg:
-            # result.info = self.bg.node[result]
-            return result
+            adjacencies = self.bg[result]
+            for key, _ in adjacencies.items():
+                for ref_key, values in self.bg[key].items():
+                    if ref_key == result:
+                        return ref_key
+            return list(self.bg[result].keys())[0]
         return None
 
     def get_vertex_by_name(self, vertex_name):
@@ -206,7 +235,8 @@ class BreakpointGraph(object):
         if vertex1 in self.bg and vertex2 in self.bg[vertex1]:
             if key is None:
                 key = min(self.bg[vertex1][vertex2])
-            return BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=self.bg[vertex1][vertex2][key]["multicolor"])
+            return BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=self.bg[vertex1][vertex2][key]["multicolor"],
+                          data=self.bg[vertex1][vertex2][key]["data"])
         return None
 
     def get_edge_by_two_vertices(self, vertex1, vertex2, key=None):
@@ -240,11 +270,11 @@ class BreakpointGraph(object):
         if vertex in self.bg:
             for vertex2, edges in self.bg[vertex].items():
                 for key, data in self.bg[vertex][vertex2].items():
+                    bg_edge = BGEdge(vertex1=vertex, vertex2=vertex2, multicolor=data["multicolor"], data=data["data"])
                     if keys:
-                        yield BGEdge(vertex1=vertex, vertex2=vertex2, multicolor=data["multicolor"]), key
+                        yield bg_edge, key
                     else:
-                        yield BGEdge(vertex1=vertex, vertex2=vertex2, multicolor=data["multicolor"])
-        return None
+                        yield bg_edge
 
     def get_edges_by_vertex(self, vertex, keys=False):
         """ Iterates over edges that are incident to supplied vertex argument in current :class:`BreakpointGraph`
@@ -258,7 +288,8 @@ class BreakpointGraph(object):
         :return: generator over edges (tuples ``edge, edge_id`` if keys specified) in current :class:`BreakpointGraph` wrapped in :class:`bg.vertex.BGEVertex`
         :rtype: ``generator``
         """
-        yield from self.__get_edges_by_vertex(vertex=vertex, keys=keys)
+        for entry in self.__get_edges_by_vertex(vertex=vertex, keys=keys):
+            yield entry
 
     def __edges_between_two_vertices(self, vertex1, vertex2, keys=False):
         """ Iterates over edges between two supplied vertices in current :class:`BreakpointGraph`
@@ -300,7 +331,8 @@ class BreakpointGraph(object):
         :return: generator over edges (tuples ``edge, edge_id`` if keys specified) between two supplied vertices in current :class:`BreakpointGraph` wrapped in :class:`bg.vertex.BGVertex`
         :rtype: ``generator``
         """
-        yield from self.__edges_between_two_vertices(vertex1=vertex1, vertex2=vertex2, keys=keys)
+        for entry in self.__edges_between_two_vertices(vertex1=vertex1, vertex2=vertex2, keys=keys):
+            yield entry
 
     def connected_components_subgraphs(self, copy=True):
         """ Iterates over connected components in current :class:`BreakpointGraph` object, and yields new instances of :class:`BreakpointGraph` with respective information deep-copied by default (week reference is possible of specified in method call).
@@ -313,7 +345,7 @@ class BreakpointGraph(object):
         for component in nx.connected_component_subgraphs(self.bg, copy=copy):
             yield BreakpointGraph(component)
 
-    def __delete_bgedge(self, bgedge, key=None):
+    def __delete_bgedge(self, bgedge, key=None, keep_vertices=False):
         """ Deletes a supplied :class:`bg.edge.BGEdge` from a perspective of multi-color substitution. If unique identifier ``key`` is not provided, most similar (from perspective of :meth:`bg.multicolor.Multicolor.similarity_score` result) edge between respective vertices is chosen for change.
 
         If no unique identifier for edge to be changed is specified, edge to be updated is determined by iterating over all edges between vertices in supplied :class:`bg.edge.BGEdge` instance and the edge with most similarity score to supplied one is chosen.
@@ -332,9 +364,7 @@ class BreakpointGraph(object):
         # candidate edges setup
         #
         ############################################################################################################
-        candidate_id = None
-        candidate_score = -1
-        candidate_data = None
+
         if key is not None:
             ############################################################################################################
             #
@@ -351,23 +381,37 @@ class BreakpointGraph(object):
                 #
                 ############################################################################################################
                 self.bg.remove_edge(v=bgedge.vertex1, u=bgedge.vertex2, key=key)
+                if keep_vertices:
+                    self.bg.add_node(bgedge.vertex1)
+                    self.bg.add_node(bgedge.vertex2)
         else:
-            for v1, v2, key, data in self.bg.edges_iter(nbunch=bgedge.vertex1, data=True, keys=True):
-                ############################################################################################################
-                #
-                # iterate over all edges and determine which edge has a multicolor most related to the provided for deletion edge
-                #
-                ############################################################################################################
-                if v2 == bgedge.vertex2:
-                    score = Multicolor.similarity_score(bgedge.multicolor, data["multicolor"])
-                    if score > candidate_score:
-                        candidate_id = key
-                        candidate_data = data
-                        candidate_score = score
+            candidate_data, candidate_id, candidate_score = self.__determine_most_suitable_edge_for_deletion(bgedge)
             if candidate_data is not None:
                 candidate_data["multicolor"] -= bgedge.multicolor
                 if len(self.bg[bgedge.vertex1][bgedge.vertex2][candidate_id]["multicolor"].multicolors) == 0:
                     self.bg.remove_edge(v=bgedge.vertex1, u=bgedge.vertex2, key=candidate_id)
+                    if keep_vertices:
+                        self.bg.add_node(bgedge.vertex1)
+                        self.bg.add_node(bgedge.vertex2)
+        self.cache_valid["overall_set_of_colors"] = False
+
+    def __determine_most_suitable_edge_for_deletion(self, bgedge):
+        candidate_id = None
+        candidate_score = -1
+        candidate_data = None
+        for v1, v2, key, data in self.bg.edges_iter(nbunch=bgedge.vertex1, data=True, keys=True):
+            ############################################################################################################
+            #
+            # iterate over all edges and determine which edge has a multicolor most related to the provided for deletion edge
+            #
+            ############################################################################################################
+            if v2 == bgedge.vertex2:
+                score = Multicolor.similarity_score(bgedge.multicolor, data["multicolor"])
+                if score > candidate_score:
+                    candidate_id = key
+                    candidate_data = data
+                    candidate_score = score
+        return candidate_data, candidate_id, candidate_score
 
     def delete_edge(self, vertex1, vertex2, multicolor, key=None):
         """ Creates a new :class:`bg.edge.BGEdge` instance from supplied information and deletes it from a perspective of multi-color substitution. If unique identifier ``key`` is not provided, most similar (from perspective of :meth:`bg.multicolor.Multicolor.similarity_score` result) edge between respective vertices is chosen for change.
@@ -542,7 +586,8 @@ class BreakpointGraph(object):
         :type guidance: iterable where each entry is iterable with colors entries
         :return: ``None``, performs inplace changes
         """
-        for v1, v2 in itertools.combinations(self.bg.nodes_iter(), 2):
+        vertex_pairs = [(edge.vertex1, edge.vertex2) for edge in self.edges()]
+        for v1, v2 in vertex_pairs:
             self.__split_all_edges_between_two_vertices(vertex1=v1, vertex2=v2, guidance=guidance,
                                                         sorted_guidance=sorted_guidance,
                                                         account_for_colors_multiplicity_in_guidance=account_for_colors_multiplicity_in_guidance)
@@ -615,7 +660,8 @@ class BreakpointGraph(object):
 
         :return: ``None``, performs inplace changes
         """
-        for v1, v2 in itertools.combinations(self.bg.nodes_iter(), 2):
+        pairs_of_vetices = [(edge.vertex1, edge.vertex2) for edge in self.edges()]
+        for v1, v2 in pairs_of_vetices:
             ############################################################################################################
             #
             # we iterate over all pairs of vertices in the given graph and merge edges between them
@@ -700,6 +746,8 @@ class BreakpointGraph(object):
         # k-break must ba valid to be applied
         #
         ############################################################################################################
+        vertices = {}
+        edge_data = {}
         if not isinstance(kbreak, KBreak):
             raise TypeError("Only KBreak and derivatives are allowed as kbreak argument")
         if not KBreak.valid_kbreak_matchings(kbreak.start_edges, kbreak.result_edges):
@@ -734,7 +782,16 @@ class BreakpointGraph(object):
         for vertex1, vertex2 in kbreak.start_edges:
             if vertex1.is_infinity_vertex and vertex2.is_infinity_vertex:
                 continue
-            self.__delete_bgedge(BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=kbreak.multicolor))
+            v1 = self.__get_vertex_by_name(vertex_name=vertex1.name)
+            vertices[v1] = v1
+            v2 = self.__get_vertex_by_name(vertex_name=vertex2.name)
+            vertices[v2] = v2
+            bgedge = BGEdge(vertex1=v1, vertex2=v2, multicolor=kbreak.multicolor)
+            candidate_data, candidate_id, candidate_score = self.__determine_most_suitable_edge_for_deletion(bgedge=bgedge)
+            data = candidate_data["data"]
+            edge_data[v1] = data
+            edge_data[v2] = data
+            self.__delete_bgedge(bgedge=bgedge, keep_vertices=True)
         for vertex_set in kbreak.start_edges:
             for vertex in vertex_set:
                 if vertex.is_infinity_vertex and vertex in self.bg:
@@ -757,7 +814,22 @@ class BreakpointGraph(object):
                 #
                 ############################################################################################################
                 continue
-            self.__add_bgedge(BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=kbreak.multicolor), merge=merge)
+            origin = kbreak.data.get("origin", None)
+            v1 = vertices.get(vertex1, vertex1)
+            v2 = vertices.get(vertex2, vertex2)
+            bg_edge = BGEdge(vertex1=v1, vertex2=v2, multicolor=kbreak.multicolor)
+            if "origin" in bg_edge.data:
+                bg_edge.data["origin"] = origin
+            if kbreak.is_a_fusion:
+                edge1_data = edge_data[v1]
+                edge2_data = edge_data[v2]
+                merged_edge_fragment_data = merge_fragment_edge_data(edge1_data["fragment"], edge2_data["fragment"])
+                result_edge_data = {}
+                recursive_dict_update(result_edge_data, edge1_data)
+                recursive_dict_update(result_edge_data, edge2_data)
+                recursive_dict_update(result_edge_data, {"fragment": merged_edge_fragment_data})
+                recursive_dict_update(bg_edge.data, result_edge_data)
+            self.__add_bgedge(bg_edge, merge=merge)
 
     def to_json(self, schema_info=True):
         """ JSON serialization method that account for all information-wise important part of breakpoint graph
@@ -871,3 +943,252 @@ class BreakpointGraph(object):
                                  "genome in its multicolor")
             result.__add_bgedge(edge, merge=merge)
         return result
+
+    def get_overall_set_of_colors(self):
+        if "overall_set_of_colors" not in self.cache_valid or not self.cache_valid["overall_set_of_colors"]:
+            self.cache["overall_set_of_colors"] = {color for bg_edge in self.edges() for color in bg_edge.multicolor.colors}
+            self.cache_valid["overall_set_of_colors"] = True
+        return self.cache["overall_set_of_colors"]
+
+    def get_genome_graph(self, color):
+        result = BreakpointGraph()
+        mc = Multicolor(color)
+        for edge in self.edges():
+            if mc <= edge.multicolor:
+                result.__add_bgedge(bgedge=BGEdge(vertex1=edge.vertex1, vertex2=edge.vertex2,
+                                                  multicolor=mc, data=edge.data))
+        return result
+
+    def get_blocks_order(self):
+        genome = self.get_overall_set_of_colors().pop()
+        result = {genome: []}
+        visited_vertices = set()
+        for vertex in self.nodes():
+            if vertex in visited_vertices:
+                continue
+            visited_vertices.add(vertex)
+            chr_type_f, fragment_part_forward = self._traverse_blocks_forward_from_vertex(vertex=vertex, visited_vertices=visited_vertices)
+            chr_type_r, fragment_part_reverse = self._traverse_blocks_reverse_from_vertex(vertex=vertex, visited_vertices=visited_vertices)
+            if chr_type_f != chr_type_r:
+                raise Exception("During the gene order sequence traversal we got a conflicted situation. "
+                                "Most probably case for this to happen is to have a genome with non-unique gene content")
+            if chr_type_f == "$":
+                fragment = fragment_part_reverse + fragment_part_forward
+            else:
+                fragment = fragment_part_forward if len(fragment_part_forward) > len(fragment_part_reverse) else fragment_part_reverse
+            result[genome].append((chr_type_f, fragment))
+        return result
+
+    def _traverse_blocks_from_vertex(self, vertex, visited_vertices, direction):
+        result = []
+        current_vertex = vertex
+        visited_vertices.add(current_vertex)
+        if current_vertex.is_irregular_vertex:
+            edge = list(self.get_edges_by_vertex(vertex=current_vertex))[0]
+            current_vertex = edge.vertex1 if edge.vertex1 != current_vertex else edge.vertex2
+            visited_vertices.add(current_vertex)
+        if current_vertex.is_tail_vertex and direction == "forward" or current_vertex.is_head_vertex and direction == "reverse":
+            result.append(("+", current_vertex.block_name))
+            current_vertex = current_vertex.mate_vertex
+            visited_vertices.add(current_vertex)
+        edge = list(self.get_edges_by_vertex(vertex=current_vertex))[0]
+        current_vertex = edge.vertex1 if edge.vertex1 != current_vertex else edge.vertex2
+        while current_vertex not in visited_vertices and current_vertex.is_regular_vertex:
+            visited_vertices.add(current_vertex)
+            if direction == "forward":
+                sign = "+" if current_vertex.is_tail_vertex else "-"
+            elif direction == "reverse":
+                sign = "-" if current_vertex.is_tail_vertex else "+"
+            else:
+                sign = "*"
+            result.append((sign, current_vertex.block_name))
+            current_vertex = current_vertex.mate_vertex
+            visited_vertices.add(current_vertex)
+            edge = list(self.get_edges_by_vertex(vertex=current_vertex))[0]
+            current_vertex = edge.vertex1 if edge.vertex1 != current_vertex else edge.vertex2
+        visited_vertices.add(current_vertex)
+        if current_vertex.is_irregular_vertex:
+            chr_type = "$"
+        else:
+            chr_type = "@"
+        if direction == "reverse":
+            result = result[::-1]
+        return chr_type, result
+
+    def _traverse_blocks_forward_from_vertex(self, vertex, visited_vertices):
+        return self._traverse_blocks_from_vertex(vertex=vertex, visited_vertices=visited_vertices, direction="forward")
+
+    def _traverse_blocks_reverse_from_vertex(self, vertex, visited_vertices):
+        return self._traverse_blocks_from_vertex(vertex=vertex, visited_vertices=visited_vertices, direction="reverse")
+
+    def _traverse_fragments_forward_from_vertex(self, vertex, visited_vertices):
+        return self._traverse_fragments_from_vertex(vertex=vertex, visited_vertices=visited_vertices, direction="forward")
+
+    def _traverse_fragments_reverse_from_vertex(self, vertex, visited_vertices):
+        return self._traverse_fragments_from_vertex(vertex=vertex, visited_vertices=visited_vertices, direction="reverse")
+
+    def has_edge(self, vertex1, vertex2):
+        return self.bg.has_edge(u=vertex1, v=vertex2)
+
+    def get_condensed_edge(self, vertex1, vertex2):
+        if not self.has_edge(vertex1=vertex1, vertex2=vertex2):
+            return None
+        result = BGEdge(vertex1=vertex1, vertex2=vertex2, multicolor=Multicolor())
+        for edge in self.__edges_between_two_vertices(vertex1=vertex1, vertex2=vertex2):
+            result.multicolor += edge.multicolor
+        return result
+
+    def get_fragments_orders(self):
+        genome = self.get_overall_set_of_colors().pop()
+        result = {genome: []}
+        visited_vertices = set()
+        ivs = (v for v in self.nodes() if v.is_irregular_vertex)
+        rvs = (v for v in self.nodes() if v.is_regular_vertex)
+        for vertex in itertools.chain(ivs, rvs):
+            if vertex in visited_vertices:
+                continue
+            chr_type_f, fragments_order_part_forward = self._traverse_fragments_forward_from_vertex(vertex=vertex,
+                                                                                                    visited_vertices=visited_vertices)
+            chr_type_r, fragments_order_part_reverse = self._traverse_fragments_reverse_from_vertex(vertex=vertex,
+                                                                                                    visited_vertices=visited_vertices)
+            if chr_type_f != chr_type_r:
+                raise Exception("During the fragment order sequence traversal we got a conflicted situation. "
+                                "Most probably case for this to happen is to have a genome with non-unique gene content")
+            if chr_type_f == "$":
+                if len(fragments_order_part_forward) == 0:
+                    fragment = fragments_order_part_reverse
+                elif len(fragments_order_part_reverse) == 0:
+                    fragment = fragments_order_part_forward
+                else:
+                    coincide = fragments_order_part_reverse[-1][0] == fragments_order_part_forward[0][0]
+                    coincide &= fragments_order_part_reverse[-1][1] == fragments_order_part_forward[0][1]
+                    if coincide:
+                        fragment = fragments_order_part_reverse[:-1] + fragments_order_part_forward
+                    else:
+                        fragment = fragments_order_part_reverse + fragments_order_part_forward
+            else:
+                fragment = fragments_order_part_forward if len(fragments_order_part_forward) > len(
+                        fragments_order_part_reverse) else fragments_order_part_reverse
+                if len(fragment) > 1 and fragment[-1][0] == fragment[0][0] and fragment[-1][1] == fragment[0][1]:
+                    fragment = fragment[:-1]
+            result[genome].append((chr_type_f, fragment))
+        return result
+
+    def _traverse_fragments_from_vertex(self, vertex, visited_vertices, direction):
+        result = []
+        current_vertex = vertex
+        current_fragment_name = None
+        current_fragment_orientation = None
+        if current_vertex.is_tail_vertex and direction == "forward" or current_vertex.is_head_vertex and direction == "reverse":
+            current_vertex = current_vertex.mate_vertex
+        elif not (current_vertex.is_irregular_vertex and current_vertex in visited_vertices):
+            visited_vertices.add(current_vertex)
+            edge = list(self.get_edges_by_vertex(vertex=current_vertex))[0]
+            fragment_names = get_from_dict_with_path(source_dict=edge.data, key="name", path=["fragment"])
+            if not isinstance(fragment_names, list):
+                fragment_names = [fragment_names]
+            fragment_orientations = self._get_fragment_to_edge_orientation(current_vertex=current_vertex, edge=edge)
+            fragment_orientations = self.update_orientation_with_direction(orientation=fragment_orientations,
+                                                                           direction=direction)
+            for name, orientation in zip(fragment_names, fragment_orientations):
+                new_encounter = current_fragment_name != name or current_fragment_orientation != name
+                if name not in [None, ""] and orientation not in [None, ""] and new_encounter:
+                    current_fragment_name = name
+                    current_fragment_orientation = orientation
+                    result.append((current_fragment_orientation, current_fragment_name))
+            current_vertex = edge.vertex1 if edge.vertex1 != current_vertex else edge.vertex2
+            visited_vertices.add(current_vertex)
+            if not current_vertex.is_irregular_vertex:
+                current_vertex = current_vertex.mate_vertex
+        while current_vertex not in visited_vertices and not current_vertex.is_irregular_vertex:
+            visited_vertices.add(current_vertex)
+            edge = list(self.get_edges_by_vertex(vertex=current_vertex))[0]
+            fragment_names = get_from_dict_with_path(source_dict=edge.data, key="name", path=["fragment"])
+            if not isinstance(fragment_names, list):
+                fragment_names = [fragment_names]
+            fragment_orientations = self._get_fragment_to_edge_orientation(current_vertex=current_vertex, edge=edge)
+            fragment_orientations = self.update_orientation_with_direction(orientation=fragment_orientations,
+                                                                           direction=direction)
+            if current_fragment_name == fragment_names[-1]:
+                fragment_names = fragment_names[::-1]
+                fragment_orientations = fragment_orientations[::-1]
+            for name, orientation in zip(fragment_names, fragment_orientations):
+                initial_state = current_fragment_name is None or current_fragment_orientation is None
+                new_encounter = current_fragment_name != name or current_fragment_orientation != orientation
+                new_encounter &= name not in [None, ""] and orientation not in [None, ""]
+                if initial_state or new_encounter:
+                    current_fragment_name = name
+                    current_fragment_orientation = orientation
+                    if current_fragment_name not in [None, ""] and current_fragment_orientation not in [None, ""]:
+                        result.append((current_fragment_orientation, current_fragment_name))
+            current_vertex = edge.vertex1 if edge.vertex1 != current_vertex else edge.vertex2
+            if current_vertex.is_irregular_vertex:
+                break
+            visited_vertices.add(current_vertex)
+            current_vertex = current_vertex.mate_vertex
+
+        visited_vertices.add(current_vertex)
+        if current_vertex.is_irregular_vertex:
+            chr_type = "$"
+        else:
+            chr_type = "@"
+        if direction == "reverse":
+            result = result[::-1]
+        return chr_type, result
+
+    @staticmethod
+    def _get_fragment_to_edge_orientation(current_vertex, edge):
+        v1, v2 = (edge.vertex1, edge.vertex2) if edge.vertex1 == current_vertex else (edge.vertex2, edge.vertex1)
+        forward_orientation = get_from_dict_with_path(source_dict=edge.data, key="forward_orientation", path=["fragment"])
+        if isinstance(forward_orientation, list):
+            return ["+" if BreakpointGraph._forward_orientation(v1, v2, orientation) else "-" for orientation in forward_orientation]
+        else:
+            return ["+" if BreakpointGraph._forward_orientation(v1, v2, forward_orientation) else "-"]
+
+    @staticmethod
+    def _forward_orientation(v1, v2, forward_orientation):
+        if forward_orientation is None:
+            return True
+        left_v = v1 not in forward_orientation or forward_orientation[0] == v1
+        right_v = v2 not in forward_orientation or forward_orientation[1] == v2
+        return left_v and right_v
+
+    @staticmethod
+    def update_orientation_with_direction(orientation, direction):
+        result = []
+        for entry in orientation:
+            if direction == "forward":
+                result.append(entry)
+            else:
+                result.append("-" if entry == "+" else "+")
+        return result
+
+
+class BGConnectedComponentFilter(object):
+    def __init__(self):
+        self.name = None
+
+    def accept_connected_component(self, cc, breakpoint_graph=None):
+        return True
+
+
+class CompleteMultiEdgeConnectedComponentFilter(BGConnectedComponentFilter):
+    def __init__(self):
+        super(CompleteMultiEdgeConnectedComponentFilter, self).__init__()
+        self.name = "Complete ME filter"
+
+    def accept_connected_component(self, cc, breakpoint_graph=None):
+        if len(list(cc.nodes())) != 2:
+            return True
+        genomes_cnt = len(breakpoint_graph.get_overall_set_of_colors())
+        edges_genomes_cnt = len({color for edge in cc.edges() for color in edge.multicolor.colors})
+        return genomes_cnt != edges_genomes_cnt
+
+
+class TwoNodeConnectedComponentFilter(BGConnectedComponentFilter):
+    def __init__(self):
+        super(TwoNodeConnectedComponentFilter, self).__init__()
+        self.name = "Two node filter"
+
+    def accept_connected_component(self, cc, breakpoint_graph=None):
+        return len(list(cc.nodes())) != 2
