@@ -220,7 +220,7 @@ class Multicolor(object):
             #
             ###############################################################################################
             splitting_multicolor = Multicolor(*multicolor.colors)
-            colors_guidance = [Multicolor(*multicolor.colors) for multicolor in guidance]
+            colors_guidance = [Multicolor(*tmp_multicolor.colors) for tmp_multicolor in guidance]
             ###############################################################################################
             #
             # since there might be different multicolors, with the same colors content
@@ -502,4 +502,4 @@ class Multicolor(object):
             raise TypeError("Multicolor can be intersected only with another Multicolor object")
         intersection_colors_core = self.colors.intersection(other.colors)
         colors_count = {color: min(self.multicolors[color], other.multicolors[color]) for color in intersection_colors_core}
-        return Multicolor(*(color for color in colors_count for _ in range(colors_count[color])))
\ No newline at end of file
+        return Multicolor(*(color for color in colors_count for _ in range(colors_count[color])))
