@@ -1,9 +1,15 @@
 # -*- coding: utf-8 -*-
 from collections import Counter
-from unittest.mock import Mock
+
+try:
+    from unittest.mock import Mock
+except ImportError:
+    from mock import Mock
+
 from bg.genome import BGGenome
 from bg.multicolor import Multicolor
-from bg.vertices import BlockVertex, InfinityVertex
+from bg.utils import dicts_are_equal
+from bg.vertices import BlockVertex, InfinityVertex, TaggedBlockVertex
 
 __author__ = "Sergey Aganezov"
 __email__ = "aganezov(at)gwu.edu"
@@ -14,7 +20,6 @@ from bg.edge import BGEdge, BGEdge_JSON_SCHEMA_JSON_KEY
 
 
 class BGEdgeTestCase(unittest.TestCase):
-
     def setUp(self):
         # heavily utilized variables
         self.genome1 = BGGenome("red")
@@ -109,6 +114,13 @@ class BGEdgeTestCase(unittest.TestCase):
         edge6 = BGEdge(vertex1=v3, vertex2=v1, multicolor=multicolor)
         self.assertNotEqual(edge1, edge6)
 
+        self.assertEqual(edge1, edge4)
+        edge4.data = {"fragment": {"name": 1}}
+        edge1.data = {"fragment": {"name": 2}}
+        self.assertNotEqual(edge1, edge4)
+        edge1.data = {"fragment": {"name": 1}}
+        self.assertEqual(edge1, edge4)
+
     def test_is_infinity_edge(self):
         # and edge is called an infinity edge if at least one of its vertices is an infinity vertex
         v1 = BlockVertex("v1")
@@ -282,7 +294,7 @@ class BGEdgeTestCase(unittest.TestCase):
             def make_object(self, data):
                 if "vertex1_json_id" not in data:
                     data["vertex1_json_id"] = 1
-                return super().make_object(data)
+                return super(BGEdgeJSONSchemeDefaultVertex1, self).make_object(data)
 
         json_object = {
             "vertex1_id": 1,
@@ -331,5 +343,54 @@ class BGEdgeTestCase(unittest.TestCase):
         with self.assertRaises(ValueError):
             BGEdge.from_json(data=json_object)
 
+    def test_initialization_empty_data_attribute(self):
+        v1 = TaggedBlockVertex("v1")
+        v2 = TaggedBlockVertex("v2")
+        multicolor = Multicolor(self.genome1)
+        edge = BGEdge(vertex1=v1, vertex2=v2, multicolor=multicolor)
+        self.assertDictEqual(edge.data, BGEdge.create_default_data_dict())
+
+    def test_initialization_non_empty_data_attribute(self):
+        v1 = TaggedBlockVertex("v1")
+        v2 = TaggedBlockVertex("v2")
+        multicolor = Multicolor(self.genome1)
+        data = {
+            "fragment": {"name": "scaffold1"}
+
+        }
+        edge = BGEdge(vertex1=v1, vertex2=v2, multicolor=multicolor, data=data)
+        self.assertDictEqual(edge.data, data)
+
+    def test_data_update(self):
+        update_source = {"fragment": {"name": "scaffold11"}}
+        edge = BGEdge(vertex1=TaggedBlockVertex("v1"), vertex2=TaggedBlockVertex("v2"),
+                      multicolor=Multicolor(self.genome1, self.genome2),
+                      data={"fragment": {"name": "scaffold2", "origin": "test"}})
+        edge.update_data(source=update_source)
+        self.assertIsInstance(edge.data, dict)
+        self.assertIn("fragment", edge.data)
+        self.assertIsInstance(edge.data["fragment"], dict)
+        self.assertIn("name", edge.data["fragment"])
+        self.assertIn("origin", edge.data["fragment"])
+        self.assertEqual(edge.data["fragment"]["name"], "scaffold11")
+        self.assertEqual(edge.data["fragment"]["origin"], "test")
+
+    def test_data_update_empty_source(self):
+        edge = BGEdge(vertex1=TaggedBlockVertex("v1"), vertex2=TaggedBlockVertex("v2"),
+                      multicolor=Multicolor(self.genome1, self.genome2),
+                      data={"fragment": {"name": "scaffold2", "origin": "test"}})
+        update_source = {}
+        edge.update_data(source=update_source)
+        self.assertTrue(dicts_are_equal(edge.data, {"fragment": {"name": "scaffold2", "origin": "test"}}))
+
+    def test_data_update_non_dict_source(self):
+        edge = BGEdge(vertex1=TaggedBlockVertex("v1"), vertex2=TaggedBlockVertex("v2"),
+                      multicolor=Multicolor(self.genome1, self.genome2),
+                      data={"fragment": {"name": "scaffold2", "origin": "test"}})
+        for source in [1, "2", Multicolor(), (1, ), [2, ]]:
+            with self.assertRaises(ValueError):
+                edge.update_data(source=source)
+
+
 if __name__ == '__main__':  # pragma: no cover
-    unittest.main()         # pragma: no cover
+    unittest.main()  # pragma: no cover
