@@ -4,6 +4,7 @@ __email__ = "aganezov(at)gwu.edu"
 __status__ = "production"
 
 import unittest
+
 from bg.vertices import BGVertex_JSON_SCHEMA_JSON_KEY, BGVertex, BlockVertex, InfinityVertex, TaggedVertex, TaggedBlockVertex, \
     TaggedInfinityVertex
 
@@ -153,7 +154,7 @@ class BlockVertexTestCase(BGVertexTestCase):
     """ Update vertex_class to call BlockVertex rather than BGVertex, and update overwritten portions of the class """
 
     def setUp(self):
-        super().setUp()
+        super(BlockVertexTestCase, self).setUp()
         self.vertex_class = BlockVertex
 
     def test_inheritance(self):
@@ -167,12 +168,37 @@ class BlockVertexTestCase(BGVertexTestCase):
         self.assertTrue(v.is_regular_vertex)
         self.assertTrue(v.is_block_vertex)
 
+    def test_is_head_vertex(self):
+        self.assertTrue(self.vertex_class("vertexh").is_head_vertex)
+        self.assertFalse(self.vertex_class("vertexl").is_head_vertex)
+
+    def test_is_tail_vertex(self):
+        self.assertFalse(self.vertex_class("vertexl").is_tail_vertex)
+        self.assertTrue(self.vertex_class("vertext").is_tail_vertex)
+
+    def test_get_vertex_mate_if_no_specified(self):
+        v = self.vertex_class(self.str_name1)
+        self.assertIsNone(v.mate_vertex)
+
+    def test_get_vertex_mate(self):
+        v = self.vertex_class(self.str_name1)
+        v1 = self.vertex_class(self.str_name2)
+        v.mate_vertex = v1
+        v1.mate_vertex = v
+        self.assertEqual(v.mate_vertex, v1)
+        self.assertEqual(v1.mate_vertex, v)
+
+    def test_block_name(self):
+        self.assertEqual(self.vertex_class("vertexh").block_name, "vertex")
+        self.assertEqual(self.vertex_class("vertext").block_name, "vertex")
+        self.assertEqual(self.vertex_class("vertex").block_name, "vertex")
+
 
 class InfinityVertexTestCase(BGVertexTestCase):
     """ Update vertex_class to call InfinityVertex rather than BGVertex, and update overwritten portions of the class """
 
     def setUp(self):
-        super().setUp()
+        super(InfinityVertexTestCase, self).setUp()
         self.vertex_class = InfinityVertex
         self.block_vertex = BlockVertex(self.str_name1)
 
@@ -192,6 +218,12 @@ class InfinityVertexTestCase(BGVertexTestCase):
         self.assertTrue(i_v.is_infinity_vertex)
         self.assertTrue(i_v.is_irregular_vertex)
 
+    def test_is_head_vertex(self):
+        self.assertFalse(self.vertex_class("vertexh").is_tail_vertex)
+
+    def test_is_tail_vertex(self):
+        self.assertFalse(self.vertex_class("vertext").is_tail_vertex)
+
     def test_inheritance(self):
         self.assertIsInstance(InfinityVertex(self.block_vertex), BGVertex)
 
@@ -264,7 +296,7 @@ class TaggedVertexTestCase(BGVertexTestCase):
     """ Update vertex_class to call InfinityVertex rather than BGVertex, and update overwritten portions of the class """
 
     def setUp(self):
-        super().setUp()
+        super(TaggedVertexTestCase, self).setUp()
         self.vertex_class = TaggedVertex
         self.tagged_vertex = TaggedVertex(self.str_name1)
 
@@ -375,7 +407,7 @@ class TaggedVertexTestCase(BGVertexTestCase):
 
 class TaggedBlockVertexTestCase(TaggedVertexTestCase, BlockVertexTestCase):
     def setUp(self):
-        super().setUp()
+        super(TaggedBlockVertexTestCase, self).setUp()
         self.vertex_class = TaggedBlockVertex
         self.tagged_block_vertex = TaggedBlockVertex(self.str_name1)
 
@@ -391,10 +423,20 @@ class TaggedBlockVertexTestCase(TaggedVertexTestCase, BlockVertexTestCase):
                                                                     for tag, value in tbv.tags])
         self.assertIn(ref_name, tbv.name)
 
+    def test_is_head_vertex(self):
+        tbv = self.vertex_class("vertexh")
+        tbv.add_tag("tag1", 1)
+        self.assertTrue(tbv.is_head_vertex)
+
+    def test_is_tail_vertex(self):
+        tbv = self.vertex_class("vertext")
+        tbv.add_tag("tag1", 1)
+        self.assertTrue(tbv.is_tail_vertex)
+
 
 class TaggedInfinityVertexTestCase(TaggedVertexTestCase, InfinityVertexTestCase):
     def setUp(self):
-        super().setUp()
+        super(TaggedInfinityVertexTestCase, self).setUp()
         self.vertex_class = TaggedInfinityVertex
         self.tagged_infinity_vertex = TaggedInfinityVertex(self.str_name1)
 
